#!/bin/bash

. /etc/socket-relay/path.conf
. /etc/socket-relay/incoming.conf

. $INCOMING_PATH/common_functions

TAG=incoming_modem
PID=$((0))

# socat
if [ "$SOCAT_DEBUG" != "" ]; then
    SOCAT_OPTIONS_DATA=$SOCAT_DEBUG
fi

if [ $SOCAT_DATA_POLL_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_DATA="$SOCAT_OPTIONS_DATA -t $SOCAT_DATA_POLL_TIMEOUT"
fi

if [ $SOCAT_DATA_INACTIVITY_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_DATA="$SOCAT_OPTIONS_DATA -T $SOCAT_DATA_INACTIVITY_TIMEOUT"
fi

# termios
if [ $TERMIOS_RAW -eq 1 ]; then
    #TERMIOS="rawer"
    TERMIOS="raw,echo=${TERMIOS_ECHO}"
else
    TERMIOS="icanon=${TERMIOS_ICANON},echo=${TERMIOS_ECHO},crtscts=${TERMIOS_CRTSCTS}"
    if [ "$TERMIOS_CS" != "" ]; then
        TERMIOS="$TERMIOS,$TERMIOS_CS"
    fi
fi

if [ $TERMIOS_RAW -eq 0 -a "$TERMIOS_BAUDRATE" != "" ]; then
    TERMIOS="$TERMIOS,$TERMIOS_BAUDRATE"
fi

signal_hup()
{
    logger -i -t $TAG "SIGNAL HUP"
    release_disconnect_modem
    exit 1
}

signal_int()
{
    logger -i -t $TAG "SIGNAL INT"
    release_disconnect_modem
    exit 1
}

signal_term()
{
    logger -i -t $TAG "SIGNAL TERM"
    release_disconnect_modem
    exit 1
}

signal_quit()
{
    logger -i -t $TAG "SIGNAL QUIT"
    release_disconnect_modem
    exit 1
}

signal_abrt()
{
    logger -i -t $TAG "SIGNAL ABRT"
    release_disconnect_modem
    exit 1
}

signal_pipe()
{
    logger -i -t $TAG "SIGNAL PIPE"
    release_disconnect_modem
    exit 1
}

send_ack()
{
    case $1 in
        tcp_ack)
            echo "ACK:TCP:${DATA_PORT}"
            ;;
        udp_ack)
            echo "ACK:UDP:${DATA_PORT}"
            ;;
        ack)
            echo "ACK:XXX:XXX"
            ;;
        nak)
            echo "NAK:XXX:XXX"
            ;;
        *)
            logger -i -t $TAG "invalid send message!"
            ;;
    esac
}

__connect_modem()
{
    local protocol=$1
    local ret

    if [ -f /var/run/${TTYDEV}.lock ]; then 
        logger -i -t $TAG "WARNING: lock file \"/var/run/${TTYDEV}.lock\" already exists"
    fi

    socat $SOCAT_OPTIONS_DATA ${protocol}:${DATA_PORT},reuseaddr,sndbuf=${SOCKET_SEND_BUFFER},rcvbuf=${SOCKET_RECV_BUFFER} file:/dev/${TTYDEV},nonblock,${TERMIOS},waitlock=/var/run/${TTYDEV}.lock &
    ret=$?

    if [ $ret -ne 0 ]; then
        logger -i -t $TAG "socat failed with $ret"
        return 1
    fi

    echo -n $! > $PID_INCOMING_DATA
    logger -i -t $TAG "connect: ${protocol}:${DATA_PORT} <--> /dev/${TTYDEV}"

    return 0
}

connect_modem()
{
    local ret

    case $DATA_PROTOCOL in
        tcp | TCP)
            __connect_modem tcp-l
            ret=$?
            if [ $ret -eq 0 ]; then
                send_ack tcp_ack
            else
                send_ack nak
            fi
            ;;
        udp | UDP)
            __connect_modem udp-l
            ret=$?
            if [ $ret -eq 0 ]; then
                send_ack udp_ack
            else
                send_ack nak
            fi
            ;;
        *)
            send_ack nak
            logger -i -t $TAG "invalid network protocol!"
            return 1
            ;;
    esac

    return $ret
}

setup_connect_modem()
{
    if [ "$EARLY_SOCKET_CONNECT" = "yes" ]; then
        connect_modem
        if [ $? -eq 0 ]; then
            if_setup $TTYDEV
        fi
    else
        if_setup
        if [ $? -eq 0 ]; then
            connect_modem
        else
            send_ack nak
        fi
    fi

    return $?
}

send_ack_for_on_demand()
{
    if [ -f $PID_INCOMING_DATA ]; then
        case $DATA_PROTOCOL in
            tcp | TCP)
                send_ack tcp_ack
                ;;
            udp | UDP)
                send_ack udp_ack
                ;;
            *)
                send_ack nak
                ;;
        esac
    else
        send_ack nak
    fi
}

check_allow_lst()
{
    local user=$1
    local ip=$2
    local port=$3

    while read list; do
        if [ "$list" = "${user}@${ip}" ]; then
            logger -i -t $TAG "accepted request command for $user from $ip port $port"
            return 0
        fi
    done < /etc/socket-relay/allow.lst

    logger -i -t $TAG "invalid user $user from $ip port $port"

    return 1
}

check_receive()
{
    local cmd=$1
    local user=$2
    local ip=$3
    local port=$4

    case $cmd in
        CONNECT | DISCONNECT)
            if [ "$ALLOW_LIST" = "yes" ]; then
                check_allow_lst $user $ip $port
                if [ $? -eq 0 ]; then
                    return 0
                fi
            else
                return 0
            fi
            ;;
        *)
            logger -i -t $TAG "invalid command: $cmd"
            ;;
    esac

    return 1
}

trap signal_hup HUP
trap signal_int INT
trap signal_term TERM
trap signal_quit QUIT
trap signal_abrt ABRT
trap signal_pipe PIPE

if [ ! -c /dev/$TTYDEV ]; then
    logger -i -t $TAG "unable to connect /dev/${TTYDEV}: No such file or directory"
    exit 1
fi

logger -i -t $TAG "starting"

while read line; do
    cmd=$(echo -n $line | awk -F":" '{print $1}')
    user=$(echo -n $line | awk -F":" '{print $2}')
    ip=$SOCAT_PEERADDR
    port=$SOCAT_PEERPORT

    logger -i -t $TAG "CMD=$cmd USER=$user IP=$ip PORT=$port"

    check_receive $cmd $user $ip $port
    if [ $? -eq 0 ]; then
        case $cmd in
            CONNECT)
                if [ -f $PID_INCOMING_DATA ]; then
                    send_ack nak
                    pid=$(cat $PID_INCOMING_DATA)
                    logger -i -t $TAG "socat is already running as $pid"
                else
                    if [ -d $ON_DEMAND_STAT ]; then
                        setup_connect_modem
                    else
                        send_ack_for_on_demand
                    fi
                fi
                ;;
            DISCONNECT)
                logger -i -t $TAG "received disconnect from $ip"
                if [ -f $PID_INCOMING_DATA ]; then
                    send_ack ack
                    sleep 1
                    release_disconnect_modem
                    break
                else
                    send_ack nak
                    logger -i -t $TAG "socat is not running"
                fi
                ;;
            *)
                send_ack nak
                ;;
        esac
    else
        send_ack nak
    fi
done

logger -i -t $TAG "end"

exit 0
