#!/bin/bash

. /etc/socket-relay/path.conf
. /etc/socket-relay/outcalling.conf

. $OUTCALLING_PATH/common_functions

TAG=outcalling_modem
DATA_IP=$1

SEND_CONNECT="CONNECT:${USER}"
SEND_DISCONNECT="DISCONNECT:${USER}"

# socat
if [ "$SOCAT_DEBUG" != "" ]; then
    SOCAT_OPTIONS_DATA=$SOCAT_DEBUG
fi

if [ $SOCAT_DATA_POLL_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_DATA="$SOCAT_OPTIONS_DATA -t $SOCAT_DATA_POLL_TIMEOUT"
fi

if [ $SOCAT_DATA_INACTIVITY_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_DATA="$SOCAT_OPTIONS_DATA -T $SOCAT_DATA_INACTIVITY_TIMEOUT"
fi

# termios
if [ $TERMIOS_RAW -eq 1 ]; then
    #TERMIOS="rawer"
    TERMIOS="raw,echo=${TERMIOS_ECHO}"
else
    TERMIOS="icanon=${TERMIOS_ICANON},echo=${TERMIOS_ECHO},crtscts=${TERMIOS_CRTSCTS}"
    if [ "$TERMIOS_CS" != "" ]; then
        TERMIOS="$TERMIOS,$TERMIOS_CS"
    fi
fi

if [ $TERMIOS_RAW -eq 0 -a "$TERMIOS_BAUDRATE" != "" ]; then
    TERMIOS="$TERMIOS,$TERMIOS_BAUDRATE"
fi

signal_hup()
{
    logger -i -t $TAG "SIGNAL HUP"
    echo $SEND_DISCONNECT
    exit 1
}

signal_int()
{
    logger -i -t $TAG "SIGNAL INT"
    echo $SEND_DISCONNECT
    exit 1
}

signal_quit()
{
    logger -i -t $TAG "SIGNAL QUIT"
    echo $SEND_DISCONNECT
    exit 1
}

signal_term()
{
    logger -i -t $TAG "SIGNAL TERM"
    echo $SEND_DISCONNECT
    exit 1
}

signal_abrt()
{
    logger -i -t $TAG "SIGNAL ABRT"
    echo $SEND_DISCONNECT
    exit 1
}

signal_pipe()
{
    logger -i -t $TAG "SIGNAL PIPE"
    echo $SEND_DISCONNECT
    exit 1
}

__connect_modem()
{
    local protocol=$1
    local port=$2
    local ret

    socat $SOCAT_OPTIONS_DATA ${protocol}:${DATA_IP}:${port},sndbuf=${SOCKET_SEND_BUFFER},rcvbuf=${SOCKET_RECV_BUFFER} file:/dev/${TTYDEV},${TERMIOS},waitlock=/var/run/${TTYDEV}.lock &
    ret=$?

    if [ $ret -ne 0 ]; then
        logger -i -t $TAG "socat failed with $ret"
        return 1
    fi

    echo -n $! > $PID_OUTCALLING_DATA
    logger -i -t $TAG "connect: ${protocol}:${DATA_IP}:${port} <--> /dev/${TTYDEV}"

    return 0
}

connect_modem()
{
    local protocol=$1
    local port=$2
    local ret

    case $protocol in
        tcp | TCP)
            __connect_modem tcp $port
            ret=$?
            if [ $ret -ne 0 ]; then
                logger -i -t $TAG "connect failed!"
            fi
            ;;
        udp | UDP)
            __connect_modem udp $port
            ret=$?
            if [ $ret -ne 0 ]; then
                logger -i -t $TAG "connect failed!"
            fi
            ;;
        *)
            logger -i -t $TAG "invalid network protocol!"
            return 1
            ;;
    esac

    return 0
}

setup_connect_modem()
{
    local protocol=$1
    local port=$2
    local ret

    if [ "$EARLY_CALLER_MODEM_SETUP" = "yes" ]; then
        connect_modem $protocol $port
        ret=$?
    else
        if [ "$EARLY_SOCKET_CONNECT" = "yes" ]; then
            connect_modem $protocol $port
            ret=$?
            if [ $ret -eq 0 ]; then
                logger -i -t $TAG "if_setup..."
                if_setup
                ret=$?
            fi
        else
            logger -i -t $TAG "if_setup..."
            if_setup
            ret=$?
            if [ $ret -eq 0 ]; then
                connect_modem $protocol $port
                ret=$?
            fi
        fi
    fi

    if [ $ret -ne 0 ]; then
        echo $SEND_DISCONNECT
        logger -i -t $TAG "connection failed!!"
    fi

    return $ret
}

trap signal_hup HUP
trap signal_int INT
trap signal_quit QUIT
trap signal_abrt TERM
trap signal_abrt ABRT
trap signal_pipe PIPE

logger -i -t $TAG "starting"

if [ ! -c /dev/$TTYDEV ]; then
    logger -i -t $TAG "unable to connect /dev/${TTYDEV}: No such file or directory"
    exit 1
fi

if [ "$EARLY_CALLER_MODEM_SETUP" = "yes" ]; then
    if [ "$EARLY_CALLER_MODEM_SETUP_EARLY_REQ" = "yes" ]; then
        echo $SEND_CONNECT
        logger -i -t $TAG "if_setup..."
        if_setup
        if [ $? -ne 0 ]; then
            echo $SEND_DISCONNECT
            logger -i -t $TAG "connection failed!!"
            exit 1
        fi
    else
        logger -i -t $TAG "if_setup..."
        if_setup
        if [ $? -ne 0 ]; then
            logger -i -t $TAG "connection failed!!"
            exit 1
        fi
        echo $SEND_CONNECT
    fi
else
    echo $SEND_CONNECT
fi

logger -i -t $TAG "wait for ACK..."
read -t $CALL_TIMEOUT line
if [ $? -gt 128 ]; then
    logger -i -t $TAG "receive timeout"
    exit 1
else
    logger -i -t $TAG "$line"
    cmd=$(echo -n $line | awk -F":" '{print $1}')
    protocol=$(echo -n $line | awk -F":" '{print $2}')
    port=$(echo -n $line | awk -F":" '{print $3}')
fi

case $cmd in
    ACK)
        mkdir $OUTCALLING_RECV_ACK
        setup_connect_modem $protocol $port
        if [ $? -ne 0 ]; then
            exit 1
        fi
        ;;
    NAK)
        logger -i -t $TAG "connection request was failed"
        exit 1
        ;;
    *)
        exit 1
        ;;
esac

logger -i -t $TAG "connection succeed!!"

while true; do
    PID=$(cat $PID_OUTCALLING_DATA)
    if [ "$PID" = "disconnect" ]; then
        echo $SEND_DISCONNECT
        logger -i -t $TAG "disconnection request to incoming server"
        sleep $DISCONNECT_WAIT
        break
    else
        sleep 1
    fi
done

logger -i -t $TAG "end"

exit 0
